package Plugins::JiveMenuTest::Plugin;


# ----------------------------------------------------------------------------
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
# ----------------------------------------------------------------------------



use strict;
use base qw(Slim::Plugin::Base);
use Slim::Utils::Misc;
use XML::Simple;
use Data::Dumper;
use Slim::Utils::Log;
use Slim::Utils::Prefs;


my $thistag = "jivemenutest";


my $log = Slim::Utils::Log->addLogCategory({ 'category' => 'plugin.' . $thistag, 'defaultLevel' => 'INFO', 'description'  => 'description here' });


sub debug
{
	my $txt = shift;
	$log->info($txt . "\n");
}



# ------ slimserver delegates and initialization ------

my %noFunctions = ();

sub getFunctions
{
	return \%noFunctions;
}

sub getDisplayName
{
	return 'test';
}

sub enabled
{
	return 1;
}

sub initPlugin
{
	my $class = shift;
	$class->SUPER::initPlugin( @_ );

	# Register json/CLI functions
	#                                                                                    |requires Client
	#                                                                                    |  |is a Query
	#                                                                                    |  |  |has Tags
	#                                                                                    |  |  |  |Function to call
	#                                                                                    C  Q  T  F
	Slim::Control::Request::addDispatch([$thistag . '.submenu', '_index', '_quantity'], [1, 1, 1, \&submenuCommand]);

	# Subscribe to player connect/disconnect messages
	Slim::Control::Request::subscribe( \&clientEvent, [['client'],['new','reconnect','disconnect']]	);
}

sub shutdown
{
	Slim::Control::Request::unsubscribe( \&clientEvent );
}



my $doneJiveInit = 0;

sub clientEvent {
	my $request = shift;
	my $client  = $request->client;
	
	if ( !defined $client ) {
		return;
	}
	
	if( !$doneJiveInit )
	{
		# Register the top-level jive node
		my $node = {
				id => $thistag,
				text => "Top Menu",
				weight => 5,
				node => 'extras',
		};
		Slim::Control::Jive::registerPluginNode( $node );
		debug( $thistag . " node registered" );
		$doneJiveInit = 1;
	}

	my @menuItems = jiveTopMenu( $client );
	Slim::Control::Jive::registerPluginMenu( \@menuItems, $thistag, $client );
}



# ------ main-menu mode ------



sub jiveTopMenu
{
	my $client = shift;
	my $request = shift;

	debug( "jiveTopMenu" );
	my @menuItems = ();

	push @menuItems, {
		text => "Item One",
		id => $thistag . "1",
		weight => 10,
		actions => {
			go => {
				player => 0,
				cmd => [ $thistag . '.submenu' ],
				params => { 'menuitem_id' => "1" },
			},
		},
	};

	push @menuItems, {
		text => "Item Two",
		id => $thistag . "2",
		weight => 20,
		actions => {
			go => {
				player => 0,
				cmd => [ $thistag . '.submenu' ],
				params => { 'menuitem_id' => "2" },
			},
		},
	};

	push @menuItems, {
		text => "Item Three",
		id => $thistag . "3",
		weight => 30,
		actions => {
			go => {
				player => 0,
				cmd => [ $thistag . '.submenu' ],
				params => { 'menuitem_id' => "3" },
			},
		},
	};

	push @menuItems, {
		text => "Item Four",
		id => $thistag . "4",
		weight => 40,
		actions => {
			go => {
				player => 0,
				cmd => [ $thistag . '.submenu' ],
				params => { 'menuitem_id' => "4" },
			},
		},
	};

	return @menuItems;
}



sub jiveSubMenu
{
	my $client = shift;
	my $request = shift;

	my $item = $request->getParam('menuitem_id');

	debug( "jiveSubMenu " . $item );
	my @menuItems = ();

	push @menuItems, {
		text => $item . "Sub Item One",
		id => $thistag . $item . "sub" . "1",
		weight => 10,
		actions => {
			go => {
				player => 0,
				cmd => [ $thistag . '.submenu' ],
				params => { 'menuitem_id' => "1" },
			},
		},
	};

	push @menuItems, {
		text => $item . "Sub Item Two",
		id => $thistag . $item . "sub" . "2",
		weight => 20,
		actions => {
			go => {
				player => 0,
				cmd => [ $thistag . '.submenu' ],
				params => { 'menuitem_id' => "2" },
			},
		},
	};

	push @menuItems, {
		text => $item . "Sub Item Three",
		id => $thistag . $item . "sub" . "3",
		weight => 30,
		actions => {
			go => {
				player => 0,
				cmd => [ $thistag . '.submenu' ],
				params => { 'menuitem_id' => "3" },
			},
		},
	};

	push @menuItems, {
		text => $item . "Sub Item Four",
		id => $thistag . $item . "sub" . "4",
		weight => 40,
		actions => {
			go => {
				player => 0,
				cmd => [ $thistag . '.submenu' ],
				params => { 'menuitem_id' => "4" },
			},
		},
	};

	return @menuItems;
}





# ----- The CLI requests -----


sub submenuCommand
{
	my $request = shift;
	my $client = $request->client();
	debug( "command: submenu" );

	if( $request->isNotQuery([[$thistag . '.submenu']]) )
	{
		my $client = $request->client();
		oops( $client, undef, "submenu not command" );
		$request->setStatusBadDispatch();
		return;
	}

	my @menuItems = jiveSubMenu( $client, $request );
	debug(Data::Dump::dump(@menuItems));

	my $numitems = scalar(@menuItems);
	$request->addResult("count", $numitems);
	$request->addResult("offset", 0);
	my $cnt = 0;
	for my $eachmenu (@menuItems[0..$numitems]) {
		$request->setResultLoopHash('item_loop', $cnt, $eachmenu);
		$cnt++;
	}
	
	$request->setStatusDone();
	debug( "submenu done" );
}




1;
